@@ -110,7 +110,75 @@ class MultiBulkStorage(object):
         self.items.append(item)
 
 
-class RedisProtocol(basic.LineReceiver, policies.TimeoutMixin):
+class LineReceiver(protocol.Protocol, basic._PauseableMixin):
+    line_mode = 1
+    __buffer = ''
+    delimiter = '\r\n'
+    MAX_LENGTH = 16384
+
+    def clearLineBuffer(self):
+        b = self.__buffer
+        self.__buffer = ""
+        return b
+
+    def dataReceived(self, data, unpause=False):
+        if unpause is True:
+            if self.__buffer:
+                self.__buffer = data + self.__buffer
+            else:
+                self.__buffer += data
+
+            self.resumeProducing()
+        else:
+            self.__buffer = self.__buffer + data
+
+        while self.line_mode and not self.paused:
+            try:
+                line, self.__buffer = self.__buffer.split(self.delimiter, 1)
+            except ValueError:
+                if len(self.__buffer) > self.MAX_LENGTH:
+                    line, self.__buffer = self.__buffer, ''
+                    return self.lineLengthExceeded(line)
+                break
+            else:
+                linelength = len(line)
+                if linelength > self.MAX_LENGTH:
+                    exceeded = line + self.__buffer
+                    self.__buffer = ''
+                    return self.lineLengthExceeded(exceeded)
+                why = self.lineReceived(line)
+                if why or self.transport and self.transport.disconnecting:
+                    return why
+        else:
+            if not self.paused:
+                data = self.__buffer
+                self.__buffer = ''
+                if data:
+                    return self.rawDataReceived(data)
+
+    def setLineMode(self, extra=''):
+        self.line_mode = 1
+        if extra:
+            self.pauseProducing()
+            reactor.callLater(0, self.dataReceived, extra, True)
+
+    def setRawMode(self):
+        self.line_mode = 0
+
+    def rawDataReceived(self, data):
+        raise NotImplementedError
+
+    def lineReceived(self, line):
+        raise NotImplementedError
+
+    def sendLine(self, line):
+        return self.transport.write(line + self.delimiter)
+
+    def lineLengthExceeded(self, line):
+        return self.transport.loseConnection()
+
+
+class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     """
     Redis client protocol.
     """
@@ -154,8 +222,8 @@ class RedisProtocol(basic.LineReceiver, policies.TimeoutMixin):
     def connectionLost(self, why):
         self.connected = 0
         self.factory.delConnection(self)
-        basic.LineReceiver.connectionLost(self, why)
-        while self.replyQueue.pending:
+        LineReceiver.connectionLost(self, why)
+        while self.replyQueue.waiting:
             self.replyReceived(ConnectionError("Lost connection"))
 
     def lineReceived(self, line):
@@ -244,7 +312,7 @@ class RedisProtocol(basic.LineReceiver, policies.TimeoutMixin):
             bulk_buffer = "".join(self.bulk_buffer)[:-2]
             self.bulk_buffer = []
             self.bulkDataReceived(bulk_buffer)
-            reactor.callLater(0, self.setLineMode, extra=rest)
+            self.setLineMode(extra=rest)
 
     def bulkDataReceived(self, data):
         """
@@ -904,7 +972,7 @@ class RedisProtocol(basic.LineReceiver, policies.TimeoutMixin):
             pieces = [cmd, key, min, max, "WITHSCORES"]
         else:
             pieces = [cmd, key, min, max]
-        if offset and count:
+        if offset is not None and count is not None:
             pieces.extend(("LIMIT", offset, count))
         r = self.execute_command(*pieces)
         if withscores:
@@ -1235,9 +1303,6 @@ class MonitorProtocol(RedisProtocol):
     take care with the performance impact: http://redis.io/commands/monitor
     """
 
-    def connectionLost(self, why):
-        pass
-
     def messageReceived(self, message):
         pass
 
@@ -1252,9 +1317,6 @@ class MonitorProtocol(RedisProtocol):
 
 
 class SubscriberProtocol(RedisProtocol):
-    def connectionLost(self, why):
-        pass
-
     def messageReceived(self, pattern, channel, message):
         pass
 
@@ -1263,7 +1325,10 @@ class SubscriberProtocol(RedisProtocol):
             if reply[-3] == u"message":
                 self.messageReceived(None, *reply[-2:])
             else:
+                self.replyQueue.put(reply[-3:])
                 self.messageReceived(*reply[-3:])
+        elif isinstance(reply, Exception):
+            self.replyQueue.put(reply)
 
     def subscribe(self, channels):
         if isinstance(channels, (str, unicode)):
@@ -1285,19 +1350,6 @@ class SubscriberProtocol(RedisProtocol):
             patterns = [patterns]
         return self.execute_command("PUNSUBSCRIBE", *patterns)
 
-
-class SubscriberFactory(protocol.ReconnectingClientFactory):
-    maxDelay = 120
-    continueTrying = True
-    protocol = SubscriberProtocol
-
-
-class MonitorFactory(protocol.ReconnectingClientFactory):
-    maxDelay = 120
-    continueTrying = True
-    protocol = MonitorProtocol
-
-
 class ConnectionHandler(object):
     def __init__(self, factory):
         self._factory = factory
@@ -1618,6 +1670,19 @@ class RedisFactory(protocol.ReconnectingClientFactory):
 
         raise RedisError("In transaction")
 
+class SubscriberFactory(RedisFactory):
+    protocol = SubscriberProtocol
+
+    def __init__(self, isLazy=False, handler=ConnectionHandler):
+        RedisFactory.__init__(self, None, None, 1, isLazy=isLazy,
+                              handler=handler)
+
+class MonitorFactory(RedisFactory):
+    protocol = MonitorProtocol
+
+    def __init__(self, isLazy=False, handler=ConnectionHandler):
+        RedisFactory.__init__(self, None, None, 1, isLazy=isLazy,
+                              handler=handler)
 
 def makeConnection(host, port, dbid, poolsize, reconnect, isLazy):
     uuid = "%s:%s" % (host, port)
@@ -1767,4 +1832,4 @@ __all__ = [
 ]
 
 __author__ = "Alexandre Fiori"
-__version__ = version = "0.8"
+__version__ = version = "0.9"
